
/* The rpn expression evaluator.  Returns top of stack on success,
 * arbitrary value on error.
 */
double rpn_expr(double initval,	/* Pushed onto top of stack if RPN_INIT is
				 * in flags; otherwise ignored.
				 */
		int flags,	/* See below */
		char *exprs[],	/* Array of strings that make up total
				 * expression to process; equivalent to
				 * concatenating strings with spaces between
				 * them.
				 */
		int nexpr,	/* number of elements in exprs. */
		char *prefix,	/* Print <prefix: > before any error message */
		char *msgbuffer); /* Successful return: *msgbuffer='\0'.
				   * Error return: message in msgbuffer,
				   *	less than 200 chars long.
				   */
 
/* To execute rpn commands out of a file: */
double
rpn_file(double val,	/* passed to rpn_expr() */
	int flags,	/* passed to rpn_expr() */
	char *tag,	/* Referred to in error messages; typically filename */
	FILE *fp,	/* Either open file to process, or NULL, meaning
			 * to open and process the file named by tag.
			 */
	int doprompt,	/* Give prompt before reading each line? */
	char *prefix,	/* passed to rpn_expr() */
	char *msg);	/* passed to rpn_expr() */

void *rpn_macros();	/* Returns ptr to the hash table of macro strings */

double *rpn_stack();	/* Returns pointer to the rpn expression stack */

char *rpn_print(char *s, double x);	/* Prints x using the last-recorded
					 * printing information.  Returns s.
					 * Prints to stdout if s is null;
					 * prints into s otherwise.
					 */

/* Normally, rpn_file() uses fgets() is used for input.  You can supply an
 * alternative function (that of course has the same interface as fgets) here,
 * and it will be called instead.  If you want to restore fgets() in
 * place of your own function, use   rpn_fgets = &fgets;
 */
extern char * (*rpn_fgets)(char *buf, int len, FILE *fp);

void *rpn_variables();	/* Returns ptr to the hash table of variables */

     /* The flags that can be used in rpn_expr are or'd together from: */
	/*      RPN_INIT (clear stack & do init by pushing arg initval;
	 *              otherwise, stack isn't changed on entry).
	 *      RPN_AUTOPRINT (print top of stack just before a success
	 *              return, unless last cmd printed it).
	 *	RPN_ECHO (echo each input expression before executing).
	 *	RPN_NOFILEOK (don't complain if rpn_file() is asked to process
	 *		a file that doesn't exist.  Useful for handling
	 *		rc files.
	 */

#define RPN_INIT	001
#define RPN_AUTOPRINT	002
#define RPN_ECHO	004
#define RPN_NOFILEOK	010
