#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include "expressions.h"
#include "Hsearch.h"

static char *prog;

int
main(int argc, char **argv)
{
    extern char *optarg;
    extern int optind;
    char *p, rcfile[1024], buf[500], msg[500], *s;
    void *macros, *variables;
    double x;
    int c, i, nexpr;
    char *getenv(char *);
    FILE *fp;
    char *filearg=NULL;
    void print_macro(FILE *fp, int indx, char *key, char *data);
    void print_var(FILE *fp, int indx, char *key, char *data);

    prog = (s=strrchr(argv[0], '/')) ? s+1 : argv[0];
    printf("%s %s.%s\n", prog, VERSION, PATCHLEVEL);

    while ((c = getopt(argc, argv, "f:h")) != EOF) {
	switch(c) {
	case 'f':
	    filearg = optarg;
	    break;
	case 'h':
	    printf("An RPN calculator.\n"
			"Use:\n"
			"\t%s -f file\n"
			"or\n"
			"\t%s [expr]\n", prog, prog);
	    printf("(N.B.  You can list all operators by entering 'h')\n");
	    return 0;
	    break;
	}
    }

    /* Process rcfile first.  Note that we overwrite the contents of the
     * file at program exit.
     */
    s = getenv("HOME");
    if (s) {
	sprintf(rcfile, "%s/.rpnrc", s);
	errno = 0;
	x = rpn_file(0., RPN_NOFILEOK, rcfile, NULL, 0, prog, msg);
	if (*msg) {
	    puts(msg);
	    if (errno)
		perror("");
	}
    } else {
	x = 0.0;
	*rcfile = '\0';
    }

    /* Process command line (filename; expression; or no args = interactive) */
    if (filearg) {
	/* Open file if it exists */
	struct stat statbuf;
	if (stat(filearg, &statbuf) == -1) {
	    if (errno == ENOENT) {
		/* No such file */
		;
	    } else {
		fprintf(stderr, "%s: failed to stat %s: ", prog, filearg);
		perror("");
		return 1;
	    }
	} else if ((fp = fopen(filearg, "r")) == NULL) {
	    fprintf(stderr,
		"%s: failed to open %s for processing: ", prog, filearg);
	    perror("");
	    return 1;
	} else {
	    x = rpn_file(x, RPN_AUTOPRINT, filearg, fp, 1, prog, msg);
	}
    } else if (argc > 1) {
	x = rpn_expr(x, RPN_AUTOPRINT, &argv[optind], argc-optind, prog, msg);
	if (*msg)
	    puts(msg);
    } else {
	x = rpn_file(x, RPN_AUTOPRINT, "<stdin>", stdin, 1, prog, msg);
    }

    /* Finally, write out macros and variables to rcfile */
    if (*rcfile && (fp = fopen(rcfile, "w")) == NULL) {
	fprintf(stderr,
		"Failed to open %s for writing out macros and variables: ",
								rcfile);
	perror("");
    } else {
	fprintf(fp, "#\n");
	fprintf(fp, "# Warning:   Automatically generated / "
					"automatically overwritten file!\n");
	fprintf(fp, "# This file is automatically overwritten by %s,"
					" which writes out\n", prog);
	fprintf(fp, "# its current macros and variables "
					"tables at exit time.\n");
	fprintf(fp, "\n#\n# Macros\n#\n");
	Hprint(rpn_macros(), fp, print_macro);
	fprintf(fp, "\n#\n# Variables\n#\n");
	Hprint(rpn_variables(), fp, print_var);
    }
    return 0;
}

void
print_macro(FILE *fp, int indx, char *key, char *data)
{
    fprintf(fp, "%s %s\n", key, data);
}

void
print_var(FILE *fp, int indx, char *key, char *data)
{
    fprintf(fp, "%-30.20g %s\tpop\n", *(double *) data, key);
}
